import argparse, sys
import notes

def addnote(args):
    note = notes.Note(nb=args.notebook)
    note.create_new_note()

def listnotes(args):
    ns = notes.Notes(args.listcnt)
    print ns.listnotes()

def simple_search(args):
    ns = notes.Notes()
    print ns.simple_search(args.targets, args.incremental)

def complex_search(args):
    if args.tag != None:
        print 'search tags:', args.tag
    if args.title != None:
        print 'search titles:', args.title
    if args.content != None:
        print 'search content:', args.content
    if args.notebook != None:
        print 'search in notebook:', args.notebook

def viewnote(args):
    note = notes.Note()
    note.view_note(args.listno)

def editnote(args):
    note = notes.Note()
    note.edit_note(args.listno)

def deletenote(args):
    note = notes.Note()
    note.delete_note(args.listno)
    
def importnotes(args):
    import evernote
    evernote.importnotes(args.evernote_file, args.notebook)

def sync_notes(args):
    import filesync
    if args.restore:
        filesync.restore()
        return
    if args.cloud:
        import cloudsync
        cloudsync.syncfiles(False, False)
        return
    filesync.backup(args.upload)

def def_parser():
    parser = argparse.ArgumentParser(prog='an',
            epilog='Use "an <cmd> -h" for the exact help of <cmd>.',
            description='ANote is a personal CLI note-taking app.')
    subparsers = parser.add_subparsers()
    parser_addnote = subparsers.add_parser('a', help='Add note')
    parser_addnote.add_argument('notebook', nargs='?', default='j',
            help='create the note in which notebook?')
    parser_addnote.set_defaults(func=addnote)

    parser_listnote = subparsers.add_parser('l',
            formatter_class=argparse.ArgumentDefaultsHelpFormatter,
            help='List recent modified notes')
    parser_listnote.add_argument('listcnt', nargs='?', type=int, default=5,
            help='how many notes in output list?')
    parser_listnote.set_defaults(func=listnotes)

    parser_searchnote = subparsers.add_parser('s', help='Simple search')
    parser_searchnote.add_argument('targets', nargs='+',
            help='the search targets in all text.')
    parser_searchnote.add_argument('-i', '--incremental', action='store_true',
            help='search in the previous search result')
    parser_searchnote.set_defaults(func=simple_search)

    parser_complexsearch= subparsers.add_parser('sc', help='Complex search')
    parser_complexsearch.add_argument('-t', '--tag', nargs='*',
            help='search in the note tags')
    parser_complexsearch.add_argument('-l', '--title', nargs='*',
            help='search in the note titles')
    parser_complexsearch.add_argument('-c', '--content', nargs='*',
            help='search in the note content')
    parser_complexsearch.add_argument('-b', '--notebook', nargs='?',
            help='in which notebook?')
    parser_complexsearch.set_defaults(func=complex_search)

    parser_viewnote = subparsers.add_parser('v', help='View the note')
    parser_viewnote.add_argument('listno', nargs='?', type=int, default=1,
            help='the order in the last note list by list or search command')
    parser_viewnote.set_defaults(func=viewnote)

    parser_editnote = subparsers.add_parser('e', help='Edit the note')
    parser_editnote.add_argument('listno', nargs='?', type=int, default=1,
            help='the order in the last note list by list or search command')
    parser_editnote.set_defaults(func=editnote)

    parser_deletenote = subparsers.add_parser('del', help='Delete the note')
    parser_deletenote.add_argument('listno', nargs='?', type=int, default=1,
            help='the order in the last note list by list or search command')
    parser_deletenote.set_defaults(func=deletenote)

    parser_importnotes = subparsers.add_parser('import', help='Import notes')
    parser_importnotes.add_argument('evernote_file',
            help='the .enex file exported from evernote')
    parser_importnotes.add_argument('notebook',
            help='the destination notebook')
    parser_importnotes.set_defaults(func=importnotes)

    parser_syncnote = subparsers.add_parser('y',
            help='Backup notes to archive file')
    group = parser_syncnote.add_mutually_exclusive_group()
    group.add_argument('-u', '--upload', action='store_true',
            help='Upload archive file to cloud server')
    group.add_argument('-r', '--restore', action='store_true',
            help='restore from local archive')
    group.add_argument('-c', '--cloud', action='store_true',
            help='sync with files on dropbox server')
    parser_syncnote.set_defaults(func=sync_notes)
    return parser

def main():
    parser = def_parser()
    args = parser.parse_args()
    args.func(args)



import sys, time, shutil, os, glob, datetime, subprocess
from settings import (note_base, lst_file, notebooks, 
                      valid_nb, invalid_nb, repo, trash_path)

cache_file = note_base + '.last-result'
old_backuped = note_base + 'old-backup/'

class Notes:
    def __init__(self, cnt=None):
        self.listcnt = cnt

    def post_process(self, paths):
        '''Save result to cache file and create user-friendly report'''
        with open(cache_file, 'w') as f:
            f.write(paths)
        return self.create_report(paths)

    def create_report(self, paths):
        if len(paths)==0:
            return 'Nothing'
        report = [Note.title_line]
        listno = 1
        for path in paths.split(): 
            note = Note(path=path)
            report.append(str(listno) + '. ' + ' '.join(note.listAttr()))
            listno = listno + 1
        return '\n'.join(report)

    def listnotes(self):
        if not os.path.exists(repo):
            sys.exit("Note repo does not exist")
        all_mkd = glob.glob(repo+'*.mkd')
        if len(all_mkd)==0:
            return "Nothing"
        list_res = subprocess.Popen(['ls', '-t'] + all_mkd, 
                        stdout=subprocess.PIPE)
        recent = subprocess.Popen(('head -'+str(self.listcnt)).split(),
                               stdin=list_res.stdout, stdout=subprocess.PIPE)
        most_recent = recent.communicate()[0]
        return self.post_process(most_recent)

    def nonincr_search(self, targets):
        first_cmd = subprocess.Popen(['grep', '-i', '-l', targets.pop(0)] + \
                 glob.glob(repo+'*.mkd'), stdout=subprocess.PIPE)
        cmd_queue = [first_cmd]
        while len(targets)>0:
            cmd = subprocess.Popen(['xargs','grep','-i','-l',targets.pop(0)],
                 stdin=cmd_queue[-1].stdout, stdout=subprocess.PIPE)
            cmd_queue.append(cmd)
        notes = cmd_queue[-1].communicate()[0]
        result = self.post_process(notes)
        return result

    def simple_search(self, targets, incremental):
        if incremental and os.path.exists(cache_file):
            first_cmd = subprocess.Popen(
                    ['xargs', 'grep', '-i', '-l', targets.pop(0)],
                    stdin=subprocess.PIPE, stdout=subprocess.PIPE,
                    stderr=subprocess.STDOUT)
            cmd_queue = [first_cmd]
            while len(targets)>0:
                cmd = subprocess.Popen(['xargs','grep','-i','-l',targets.pop(0)],
                         stdin=cmd_queue[-1].stdout, stdout=subprocess.PIPE)
                cmd_queue.append(cmd)
            with open(cache_file, 'r') as f:
                files = f.readlines()
            scope = ''.join(files)
            result = cmd_queue[-1].communicate(input=scope)[0]
            final_res = self.post_process(result)
            return final_res
        else:
            return self.nonincr_search(targets)

class Note:
    def __init__(self, nb=None, path=None):
        self.notebook = nb
        self.path = path

    def create_new_note(self):
        if not valid_nb(self.notebook):
            sys.exit(invalid_nb)
        if not os.path.exists(note_base):
            os.makedirs(note_base)
            os.makedirs(repo)
        tmp_note = note_base + 'newnote.tmp.mkd'
        created = time.strftime('%Y-%m-%d %H:%M:%S')
        created_file = time.strftime('%y%m%d%H%M%S')
        note_template = 'Title: \nTags: \nNotebook: %s [t/j/o/y/c]\n'\
                        'Created: %s\n\n------\n\n' % (self.notebook, created)
        with open(tmp_note, 'w') as f:
            f.write(note_template)
        os.system('vi ' + tmp_note)
        self.path = tmp_note
        note_attr = self.listAttr()
        if len(note_attr[1])==0:
           sys.exit('Cancel note creattion for empty title')
        if not os.path.exists(repo):
            os.makedirs(repo)
        notebook_abbr = note_attr[3][0].lower()
        shutil.move(tmp_note, repo + notebook_abbr + created_file + '.mkd')

    title_line = 'No. Updated, Title, Tags, Notebook, Created, Sync?'

    def listAttr(self):
        attr = []
        updated = os.path.getmtime(self.path)
        lmt = datetime.datetime.fromtimestamp(updated)
        attr.append(lmt.strftime('%y-%m-%d %H:%M'))
        with open(self.path, 'r') as f:
            content = f.readlines()

        title_line = content[0]
        if not title_line.startswith('Title: '):
            sys.exit('Bad note file format: 1st line should start with "Title:"')
        title = title_line[len('Title: '):-1] 
        attr.append(title)

        tags_line = content[1]
        if not tags_line.startswith('Tags: '):
            sys.exit('Bad note file format: 2nd line should start with "Tags:"')
        tags = '[' + tags_line[len('Tags: '):-1] + ']'
        attr.append(tags)

        nb_line = content[2]
        if not nb_line.startswith('Notebook: '):
            sys.exit('Bad note file format: 3rd line should start with "Notebook:"')
        nb_name = notebooks[nb_line[10]] 
        attr.append(nb_name)

        created_raw = os.path.basename(self.path)[1:11]
        created = created_raw[0:2]+'-'+created_raw[2:4]+'-'+created_raw[4:6]\
                +' '+created_raw[6:8]+':'+created_raw[8:10]
        attr.append(created)

        arcfiles = glob.glob(note_base + '*.bz2')
        if len(arcfiles) > 0:
            last_arcfile = sorted(arcfiles)[-1]
            la_name = os.path.basename(last_arcfile)
            lst = time.strptime(la_name, 'notes%y-%m-%d-%H.%M.%S.bz2')
            last_sync = time.mktime(lst)
            if updated > last_sync : attr.append('*')
        else:
            attr.append('?')

        return attr

    def _edit_note(self, listno, viewmode=False):
        with open(cache_file, 'r') as f:
            content = f.readlines()
        notepath = content[listno-1].strip()
        notename = os.path.basename(notepath)
        if not os.path.exists(old_backuped):
            os.makedirs(old_backuped)
        shutil.copyfile(notepath, 
                old_backuped + notename + '.bak')
        if viewmode:
            os.system('vi -R ' + notepath)
        else:
            os.system('vi ' + notepath)
        self.path = notepath
        note_attr = self.listAttr()
        nb_abbr = note_attr[3][0].lower()
        if nb_abbr != notename[0]:
            new_name = nb_abbr + notename[1:]
            shutil.copyfile(notepath, trash_path + notename)
            shutil.move(notepath, repo + new_name)
        
    def edit_note(self, listno):
        self._edit_note(listno)

    def view_note(self, listno):
        self._edit_note(listno, True)

    def delete_note(self, listno):
        with open(cache_file, 'r') as f:
            content = f.readlines()
        notepath = content[listno-1].strip()
        if not os.path.exists(trash_path):
            os.makedirs(trash_path)
        os.system('touch ' + notepath)
        shutil.move(notepath, trash_path + os.path.basename(notepath))



import tarfile, time, os, glob, shutil, sys
from settings import note_base, repo, trash_path

MAX_BACKUPS = 3 # only save last 3 archives

def backup(upload):
    filename = time.strftime('notes%y-%m-%d-%H.%M.%S.bz2')
    fullpath = note_base + filename
    with tarfile.open(fullpath, 'w:bz2') as tar:
        tar.add(repo)
        tar.add(trash_path)

    with tarfile.open(fullpath, 'r:bz2') as tar:
        print 'Create archive', filename, 'with size:',\
            os.path.getsize(fullpath), 'file number:',\
            str(len(tar.getmembers()))

    if upload:
        from dropbox.client import DropboxClient
        from settings import get_passport
        client = DropboxClient(get_passport())
        remote_repo = '/backup/'
        with open(fullpath, 'rb') as f:
            res = client.put_file(remote_repo + filename, f)
            print 'uploaded:', res
        return

    shutil.copyfile(fullpath, os.path.join(os.getcwd(), filename))

    all_archives = glob.glob(os.path.join(note_base, '*.bz2'))
    for old_archive in all_archives[:(-1 * MAX_BACKUPS)]:
        os.remove(old_archive)

    all_archives = glob.glob('*.bz2')
    for old_archive in all_archives[:(-1 * MAX_BACKUPS)]:
        os.remove(old_archive)

def restore():
    archives = glob.glob('*.bz2')
    if len(archives) == 0:
        msg = 'No archive file found in current directory.'
        sys.exit(msg)

    arcfile = sorted(archives)[-1]
    ret = os.system('find ' + note_base + ' -cnewer ' + arcfile +\
            '|grep mkd')
    if ret==0:
        rep = raw_input('There are files newer than archive, '\
                'continue restore? [y/n] ')
        if rep != 'y':
            sys.exit('Restore notes canceled by user.')

    confirm = raw_input('Restore notes from ' + arcfile + '? [y/n] ')
    if confirm != 'y':
        sys.exit('Restore notes canceled by user.')

    tmp_extract = note_base + 'tmp'
    os.system('rm -rf ' + tmp_extract)
    os.makedirs(tmp_extract)
    with tarfile.open(arcfile, 'r:bz2') as tar:
        tar.extractall(tmp_extract)
    for root, dirs, files in os.walk(tmp_extract):
        for adir in dirs:
            if adir=='repo' or adir=='trash':
                os.system('rm -rf ' + repo)
                os.system('rm -rf ' + trash_path)
                os.system(' '.join(['mv', os.path.join(root, adir), note_base]))
    os.system('rm -rf ' + tmp_extract)
    os.system('cp ' + arcfile + ' ' + note_base)

    all_archives = glob.glob(os.path.join(note_base, '*.bz2'))
    for old_archive in all_archives[:(-1 * MAX_BACKUPS)]:
        os.remove(old_archive)



import os, time, glob, shutil, sys, datetime
from dropbox.client import DropboxClient
from dropbox.datastore import (DatastoreManager, Date)
from dropbox.rest import ErrorResponse

import notes

from settings import (note_base, get_passport, repo, lst_file,
                      trash_path)

client = DropboxClient(get_passport())
remote_repo = '/repo/'
TABLE_ID = 'donno'

def _write_lst(delta_cursor):
    '''delta_cursor: str'''
    with open(lst_file, 'w') as f:
        f.write(time.strftime('%Y-%m-%d %H:%M:%S')+'\n')
        f.write(delta_cursor)

def syncfiles(only_upload, only_download):
    lst_exist = os.path.exists(lst_file)
    mroot = client.metadata(remote_repo)
    remote = len(mroot['contents'])
    if not os.path.exists(repo):
        os.makedirs(repo)
    local = len([f for f in os.listdir(repo) if f.endswith('.mkd')])
    if local>0 and (not lst_exist) and remote==0:
        _upload(glob.glob(repo+'*.mkd'))
        result = client.delta()
        _write_lst(result['cursor'])
        return

    if local>0 and (not lst_exist) and remote>0:
        print 'The LST file lost, build it manully.'
        return

    if local>0 and lst_exist and remote==0:
        print 'Error, maybe you can delete LST file and upload all notes to remote server.'
        return

    if local>0 and lst_exist and remote>0:
        if only_upload and only_download:
            sys.exit('You cannot use both these two options')

        if not (only_upload or only_download):
            _sync_with_server()
            return

        if only_upload and (not only_download):
            _upload_new_notes()
            return

        if only_download and (not only_upload):
            _download_new_notes()

    if local==0 and (not lst_exist) and remote==0:
        print 'There is nothing to sync with server. Try creating some notes and run this again.'
        return

    if local==0 and (not lst_exist) and remote>0:
        notepaths = [note['path'] for note in mroot['contents']]
        _download(notepaths, repo)
        result = client.delta()
        _write_lst(result['cursor'])
        return

    if local==0 and lst_exist and remote==0:
        print 'There is nothing to sync with server. Try creating some notes and run this again.'
        os.remove(lst_file)
        return

    if local==0 and lst_exist and remote>0:
        print 'Delete you LST file manually and run this again.'
        return

def _upload(notefiles):
    '''notefiles: [str], full path of uploaded files'''
    counter = 0
    mgr = DatastoreManager(client)
    ds = mgr.open_default_datastore()
    table = ds.get_table(TABLE_ID)
    for note in notefiles:
        with open(note, 'r') as f:
            response = client.put_file(remote_repo + os.path.basename(note),
                                f, overwrite=True)
        updated = os.path.getmtime(note)
        lmt = datetime.datetime.fromtimestamp(updated)
        fname = os.path.basename(note)
        recs = table.query(filename=fname)
        if len(recs) > 0:
            rec = recs.pop()
            rec.set('last_modified', lmt.strftime('%y-%m-%d %H:%M:%S'))
        else: # new note
            table.insert(filename=fname, 
                    last_modified=lmt.strftime('%y-%m-%d %H:%M:%S'))
        counter = counter + 1
        print str(counter), "uploaded:", response['path']
    ds.commit()

def _download(notefiles, dest_local_path):
    '''
    notefiles: [str], full path of downloaded file;
    dest_local_path: str
    '''
    counter = 0
    for notefile in notefiles:
        filename = os.path.basename(notefile)
        local_file_name = dest_local_path + filename
        with open(local_file_name, 'w') as lf:
            print 'Getting', notefile, '...'
            with client.get_file(notefile) as rf:
                lf.write(rf.read())
        mgr = DatastoreManager(client)
        ds = mgr.open_default_datastore()
        table = ds.get_table(TABLE_ID)
        recs = table.query(filename=filename)
        for rec in recs:
            last_modified = rec.get('last_modified')
        t = time.strptime(last_modified, '%y-%m-%d %H:%M:%S')
        lmt = time.mktime(t)
        os.utime(local_file_name, (lmt, lmt))
        counter = counter + 1
        print str(counter), 'download:', local_file_name

def _conflicted(local_set, remote_set):
    conflicts = {}
    conflicts['add'] = set(local_set['add']) & set(remote_set['add'])
    conflicts['del'] = set(local_set['del']) & set(remote_set['del'])
    return conflicts

def _download_new_notes():
    pass

def _upload_new_notes():
    with open(lst_file) as f:
        last_sync_time = f.readline().strip()
    local_changes = os.popen('find ' + repo + ' -newermt "' \
            + last_sync_time + '" -name "*.mkd"').read().split()
    print 'Local Changes:\n' + notes.Notes().create_report('\n'.join(local_changes))
    dosync = raw_input('Sync above notes?[y/n] ')
    if dosync!='y':
        print 'Cancel notes synchronization.'
        return
    _upload(local_changes)
    _write_lst(client.delta()['cursor'])

def _local_to_remote(local_changes):
    _upload(local_changes['add'])
    for note in local_changes['del']:
        fname = os.path.basename(note)
        try:
            client.file_delete(remote_repo + fname)
            print 'Note', fname, 'deleted on server'
        except ErrorResponse:
            print fname + ' not exists on server'
    # the record in datastore need to be deleted too

def _remote_to_local(remote_changes):
    local_cache = note_base + 'local-cache/'
    os.system('rm -rf ' + local_cache)
    os.makedirs(local_cache)
    _download(remote_changes['add'], local_cache)
    if len(glob.glob(local_cache + '*.mkd')) > 0:
        os.system('cp ' + local_cache + '*.mkd ' + repo)
    for note in remote_changes['del']:
        fname = os.path.basename(note)
        if os.path.exists(repo + fname):
            shutil.move(repo + fname, trash_path + fname)
            print 'Move note', fname, 'to trash'

def _sync_with_server():
    with open(lst_file) as f:
        last_sync_time = f.readline().strip()
        last_sync_cursor = f.readline().strip()

    local_changes = {}
    local_changes['add'] = os.popen('find ' + repo + ' -newermt "' \
            + last_sync_time + '" -name "*.mkd"').read().split()
    local_changes['del'] = os.popen('find ' + trash_path + ' -newermt "' \
            + last_sync_time + '" -name "*.mkd"').read().split()
    print 'local_changes--------', local_changes

    remote_changes = {'add':[], 'del':[]}
    remote_delta = client.delta(last_sync_cursor)
    mgr = DatastoreManager(client)
    ds = mgr.open_default_datastore()
    table = ds.get_table(TABLE_ID)
    change_entries = remote_delta['entries']
    print 'remote_change_entries-------------', change_entries
    while True:
        for remote_change in change_entries:
            print 'remote_change-------------', remote_change
            fullname = remote_change[0]
            fname = os.path.basename(fullname)
            if remote_change[1] == None:
                remote_changes['del'].append(fname)
            else:
                recs = table.query(filename=fname)
                for rec in recs:
                    last_modified = rec.get('last_modified')
                    lmt = time.strptime(last_modified, '%y-%m-%d %H:%M:%S')
                    lst = time.strptime(last_sync_time, '%Y-%m-%d %H:%M:%S')
                    if lmt > lst:
                        remote_changes['add'].append(fullname)
            print 'remote_changes-------------', remote_changes
        if not remote_delta['has_more']:
            break

    conflicts = _conflicted(local_changes, remote_changes)
    if len(conflicts['add']) > 0 or len(conflicts['del']) > 0:
        print 'Find conflicts:', conflicts
        return

    _local_to_remote(local_changes)
    _remote_to_local(remote_changes)
    _write_lst(remote_delta['cursor'])



import multiprocessing

from setuptools import setup

def readme():
    with open('README.md') as f:
        readme = f.read()
    return readme

setup(name='donno',
      version='0.1.10',
      description='A personal CLI note-taking app',
      long_description=readme(),
      classifiers=[
        'Operating System :: POSIX :: Linux',
        'Environment :: Console',
        'Programming Language :: Python :: 2.7',
        'Intended Audience :: Developers',
        'Topic :: Text Editors :: Text Processing'
      ],
      keywords='command-line, note-taking, linux',
      url='https://bitbucket.org/leechau/donno',
      author='Li Chao',
      author_email='leechau@126.com',
      license='MIT',
      packages=['donno'],
      entry_points = {
          'console_scripts': ['dn=donno.donno:main'],
      },
      install_requires=[
            "PyYAML>=3.10",
            "argparse>=1.2.1",
            "dropbox>=2.0.0",
            "nltk>=2.0.4",
            "nose>=1.3.0",
            "urllib3>=1.7.1",
            "wsgiref>=0.1.2",
      ],
      test_suite='nose.collector',
      tests_require=['nose'],
      zip_safe=False)



import os

note_base = os.path.expanduser('~/.donno/')
repo = note_base + 'repo/'
lst_file = note_base + '.last-sync-time'
trash_path = note_base + 'trash/'

notebooks = {'t': 'Tech', 'j': 'Jobs', 'o': 'Other', 'y': 'YiYang', 'c': 'Cached'}

def valid_nb(nb_name):
    return nb_name in notebooks.keys()

invalid_nb = "Invalid notebook name[t/j/o/y/c]"

def get_passport():
    if os.path.exists(note_base+'.my-key'):
        with open(note_base+'.my-key', 'r') as f:
            passport = f.read().strip()
    else:
        if not os.path.exists(note_base):
            os.makedirs(note_base)
        passport = raw_input("Now passport file found. Please input it: ")
        while len(passport)!=64:
            passport = raw_input("Now passport file found. Please input it: ")
        with open(note_base+'.my-key', 'w') as f:
            f.write(passport)
    return passport



import os, sys, time
from datetime import datetime
from datetime import timedelta
from xml.etree import ElementTree as ET
import nltk
import settings

def convert_time(src):
    normal_time = datetime.strptime(src, '%Y%m%dT%H%M%SZ') 
    local_time = normal_time + timedelta(hours=8)
    return local_time

def convert_created_time(src):
    local_cre_time = convert_time(src)
    created = local_cre_time.strftime('%Y-%m-%d %H:%M:%S')
    created_file = local_cre_time.strftime('%y%m%d%H%M%S')
    return {'filename': created_file+'.mkd', 'content': created}

def convert_modified_time(src):
    local_mod_time = convert_time(src)
    return time.mktime(local_mod_time.timetuple())

def convert_content(src):
    '''Did not treat the leading spaces problem. Fix it in vim manually'''
    intstr = src.replace('<div>', '\n')
    intstr = intstr.replace('</div>', '\n')
    intstr = nltk.clean_html(intstr)
    intstr = intstr.replace('&quot;', '"')
    intstr = intstr.replace('&apos;', "'")
    intstr = intstr.replace('&amp;', '&')
    intstr = intstr.replace('&lt;', '<')
    intstr = intstr.replace('&gt;', '>')
    return intstr


def importnotes(source_file, dest_nb):
    if not os.path.exists(source_file):
        sys.exit("Source file does not exist")
    if not settings.valid_nb(dest_nb):
        sys.exit(settings.invalid_nb)
    tree = ET.parse(source_file)
    root = tree.getroot()
    for note in root:
        for t in note.iter('title'):
            title = t.text
        tags = []
        for t in note.iter('tag'):
            tags.append(t.text)
        alltags = ';'.join(tags)
        for c in note.iter('content'):
            raw = c.text
        for c in note.iter('created'):
            created = c.text
        for u in note.iter('updated'):
            updated = u.text
        file_name = dest_nb + convert_created_time(created)['filename']
        with open(settings.repo + file_name, 'w') as f:
            f.write('Title: ' + title.encode('utf8') + '\n')
            f.write('Tags: ' + alltags.encode('utf8') + '\n')
            f.write('Notebook: ' + dest_nb + '[t/j/o/y/c]\n')
            f.write('Created: ' + convert_created_time(created)['content'] + '\n')
            f.write('\n------\n\n')
            f.write(convert_content(raw).encode('utf8'))
        last_modif = convert_modified_time(updated)
        os.utime(settings.repo + file_name, (last_modif, last_modif))
